import 'dart:async';
import 'dart:async' as prefix0;
import 'dart:convert';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:http/http.dart' as http;
import 'package:minilikewallet/config/config.dart';

abstract class BaseAuth {
  Future<User> singInWithEmailAndPassword(String email, String password);
  Future<String> createUserWithEmailAndPassword(String email, String password);
  Future<String> currentUser();

  Future<User> getCurrentUser();

  Future<String> getTokenFirebase();
  Future<void> signOut();
  Future<bool> checkExistUser(String phoneNumber);
  Future<List<String>> checkOldUserName(String phoneNumber);
  Future<bool> checkExistsUserEmail(String email);
}

class Auth implements BaseAuth {
  final FirebaseAuth firebaseAuth = FirebaseAuth.instance;

  Future<User> singInWithEmailAndPassword(String email, String password) async {
    User user = (await firebaseAuth.signInWithEmailAndPassword(
            email: email, password: password))
        .user;
    return user;
  }

  Future<String> createUserWithEmailAndPassword(
      String email, String password) async {
    User user = (await firebaseAuth.createUserWithEmailAndPassword(
            email: email, password: password))
        .user;
    var token = await user.getIdTokenResult();
    return token.token;
  }

  Future<String> currentUser() async {
    User user = await firebaseAuth.currentUser;
    return user.uid;
  }

  Future<User> getCurrentUser() async {
    User user = firebaseAuth.currentUser;
    return user;
  }

  Future<String> getTokenFirebase() async {
    User user = await firebaseAuth.currentUser;
    var token = await user.getIdTokenResult();
    return token.token;
  }

  Future<void> signOut() async {
    return firebaseAuth.signOut();
  }

  Future<bool> checkExistUser(String phoneNumber) async {
    String url = Env.apiUrl + '/checkExistUser';
    var response = await http.post(url, body: {
      'apiKey': Env.APIKEY,
      'secretKey': Env.SECRETKEY,
      'phone_number': phoneNumber
    });
    var body = json.decode(response.body);
    return body['result'];
  }

  Future<List<String>> checkOldUserName(String phoneNumber) async {
    String url = Env.OldAPI + '/checkOldUserName';
    var response = await http.post(url, body: {'phone_number': phoneNumber});
    var body = json.decode(response.body);
    return [body['statusCode'].toString(), body['firstname'], body['lastname']];
  }

  Future<bool> checkExistsUserEmail(String email) async {
    String url = Env.apiUrl + '/checkExistUserEmail';
    var response = await http.post(url, body: {
      'apiKey': Env.APIKEY,
      'secretKey': Env.SECRETKEY,
      'email': email
    });
    var body = json.decode(response.body);
    return body['result'];
  }

  Future<void> sendEmailVerification() async {
    User user = await firebaseAuth.currentUser;
    user.sendEmailVerification();
  }

  Future<bool> isEmailVerified() async {
    User user = await firebaseAuth.currentUser;
    return user.emailVerified;
  }
}
