import 'package:minilikewallet/library/crypto.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';

abstract class IConfigurationService {
  Future<void> setMnemonic(String value);
  Future<void> setupDone(bool value);
  Future<void> setPrivateKey(String value);
  Future<void> setAddress(String value);

  Future<String> getMnemonic();
  String getAddress();
  String getPrivateKey();
  bool didSetupWallet();
}

class ConfigurationService implements IConfigurationService {
  SharedPreferences _preferences;
  ConfigurationService(this._preferences);

  final storage = FlutterSecureStorage();
  CryptoEncryptInterface Encrypt;

  @override
  Future<void> setMnemonic(String value) async {
    Encrypt = CryptoEncrypt();
    String mnemonic = await Encrypt.decrypt(value);
    await _preferences.setString("mnemonic", mnemonic);
  }

  @override
  Future<void> setPrivateKey(String value) async {
    await _preferences.setString("privateKey", value);
  }

  @override
  Future<void> setAddress(String value) async {
    await _preferences.setString("addressETH", value);
  }

  @override
  Future<void> setupDone(bool value) async {
    await _preferences.setBool("didSetupWallet", value);
  }

  // gets
  @override
  Future<String> getMnemonic() async {
//    var xxx = await _preferences.getString("mnemonic");
//    print('xxx : '+ xxx.toString());
//    print('xxx2 : '+ await storage.read(key: 'mnemonic'));
//     return await storage.read(key: 'mnemonic')?? 'no';
    return await _preferences.getString("mnemonic") ??
        await storage.read(key: 'mnemonic') ??
        'no';
  }

  @override
  String getPrivateKey() {
    return _preferences.getString("privateKey");
  }

  @override
  String getAddress() {
    return _preferences.getString("addressETH");
  }

  @override
  bool didSetupWallet() {
    return _preferences.getBool("didSetupWallet") ?? false;
  }
}
