import 'dart:async';

import 'package:flutter/material.dart';
import 'package:minilikewallet/library/address_service.dart';
import 'package:minilikewallet/library/configuration_service.dart';
import 'package:minilikewallet/library/ethcontract.dart';
import 'package:minilikewallet/lock_all/component/lock_all_component.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:minilikewallet/provider/home_provider/check_tx_provider.dart';
import 'package:minilikewallet/provider/home_provider/home_provider.dart';
import 'package:minilikewallet/provider/home_provider/index_provider/index_provider.dart';
import 'package:minilikewallet/provider/home_provider/lock_all_provider/lock_all_provider.dart';
import 'package:minilikewallet/theme/widget.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

class LockAllScreen extends StatefulWidget {
  @override
  _LockAllScreen createState() => _LockAllScreen();
}

class _LockAllScreen extends State<LockAllScreen>
    with SingleTickerProviderStateMixin {
  CheckTXProvider _checkTXProvider;
  LockAllProvider _lockAll;
  HomeProvider _getLikeWallet;
  Widget buttonLock;
  BaseETH eth;
  IndexProvider _indexProvider;
  IConfigurationService configETH;
  IAddressService addressService;
  AnimationController controller;
  StreamController<double> streamController =
      StreamController<double>.broadcast();
  String getPK = '';
  String totalLock = '';

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();
    _getLikeWallet = Provider.of<HomeProvider>(context);
    _lockAll = Provider.of<LockAllProvider>(context);
    _indexProvider = Provider.of<IndexProvider>(context);
    _checkTXProvider = Provider.of<CheckTXProvider>(context, listen: true);
  }

  @override
  void initState() {
    controller =
        AnimationController(vsync: this, duration: Duration(seconds: 2))
          ..repeat();
    super.initState();
    eth = new EthContract();
    reloadBalance();
  }

  Future reloadBalance() async {
    SharedPreferences pref = await SharedPreferences.getInstance();
    configETH = new ConfigurationService(pref);
    addressService = new AddressService(configETH);
    final mnemonic = await configETH.getMnemonic();
    getPK = addressService.getPrivateKey(mnemonic);
    configETH.setPrivateKey(getPK);
    final address = await addressService.getPublicAddress(getPK);
    eth.getBalance(address: address.hex).then((balance) {
      streamController.sink.add(balance);
      totalLock = balance.toString();
      print("balance" + balance.toString());
    });
  }

  @override
  void dispose() {
    // TODO: implement dispose
    controller.dispose();
    super.dispose();
  }

  callLockAll({String getPK}) async {
    if (double.parse(totalLock) > 1.0) {
      _lockAll.loadStatus(status: 'ACTION_LOADING');
      print(eth);
      print(getPK);
      print(totalLock);
      final tx = await _lockAll.lockLikePoint(
          eth: eth, getPK: getPK, totalLock: totalLock);
      print(tx);
      final status = await _checkTXProvider.checkStatusTX(tx: tx);
      if (status) {
        controller.stop();
        _lockAll.loadStatus(status: 'ACTION_DONE');
      } else {
        _lockAll.loadStatus(status: 'ACTION_CONFIRM');
        WidgetTheme.showShortToastRed('เกิดข้อผิดพลาด');
      }
    } else {
      WidgetTheme.showShortToastRed('ล็อคขั้นต่ำ 1 LIKE');
    }
  }

  done() async {
    await _lockAll.loadStatus(status: 'ACTION_CONFIRM');
    controller.stop();
    await _getLikeWallet.setLockAll(status: false);
    await _indexProvider.loadBalance();
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return Column(
      children: [
        SizedBox(height: 20.h),
        Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            FlatButton(
                onPressed: () {
                  _getLikeWallet.setMenu(status: true);
                  _getLikeWallet.setLockAll(status: false);
                },
                child: WidgetTheme.backButton()),
          ],
        ),

        // SizedBox(height: 55.h),
        LockAllComponent.textAmount(amount: streamController),
        SizedBox(height: 4.h),
        LockAllComponent.text(text: 'Likepoint'),
        LockAllComponent.text(text: 'will be locked.'),
        SizedBox(height: 39.h),
        SizedBox(
          width: 769.w,
          child: LockAllComponent.text(
              text:
                  'You won’t be able to use these Likepoint until you unlock it.'),
        ),
        SizedBox(height: 144.h),
        button(),
      ],
    );
  }

  Widget button() {
    // if (_indexProvider.getBalance > 1) {
    if (_lockAll.getLoading == 'ACTION_CONFIRM') {
      buttonLock = InkWell(
          onTap: () => callLockAll(getPK: getPK),
          child: LockAllComponent.buttonAction(text: 'Confirm'));
    } else if (_lockAll.getLoading == 'ACTION_LOADING') {
      buttonLock = loading();
    } else if (_lockAll.getLoading == 'ACTION_DONE') {
      buttonLock = InkWell(
          onTap: () => done(),
          child: LockAllComponent.buttonAction(text: 'Done!'));
    }
    // }
    // else {
    //   WidgetTheme.showShortToastRed('ล็อคขั้นต่ำ 1 LIKE');
    // }

    return buttonLock;
  }

  Widget loading() {
    return Stack(
      alignment: Alignment.center,
      children: [
        LockAllComponent.buttonLoadLock(),
        AnimatedBuilder(
            animation: controller,
            builder: (_, child) {
              return Transform.rotate(
                angle: controller.value * 6,
                child: child,
              );
            },
            child: LockAllComponent.iconLoading()),
      ],
    );
  }
}
