import 'dart:async';
import 'dart:io';
import 'package:device_apps/device_apps.dart';
import 'package:flutter/services.dart';
import 'package:flutter_appavailability/flutter_appavailability.dart';
import 'package:launch_review/launch_review.dart';
import 'package:open_appstore/open_appstore.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';

abstract class OpenUrl {
  openURLAPP({String url});
  launchURL();
}

class IOpenUrl implements OpenUrl {
  openURLAPP({String url}) async {
    print(url);
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      LaunchReview.launch(androidAppId: url, iOSAppId: url);
      throw 'Could not launch $url';
    }
  }

  launchURL() async {
    String url;
    //ตรวจสอบ platform
    if (Platform.isAndroid) {
      url = 'likewallet.likewallet';
      //ตรวจสอบว่ามีการติดตั้งไว้ในเครื่องหรือยัง
      bool isInstalled = await DeviceApps.isAppInstalled(url);
      //ถ้าติดตั้งแอพแล้วให้เปิดเลย
      if (isInstalled) {
        print('open app');
        await DeviceApps.openApp(url);
      } else {
        //ถ้ายังไม่มีให้ไปเปิดใน play store เพื่อติดตั้งแทน
        print('ถ้ายังไม่มีให้ไปเปิดใน');
        openURLAPP(url: url);
      }
      //ถ้าเป็น iOS
    } else if (Platform.isIOS) {
      String os = Platform.operatingSystem;

      if (os == "ios") {
        print("Container clicked");
        try {
          await launch("likewallet://");
        } on PlatformException catch (e) {
          OpenAppstore.launch(
              androidAppId: "likewallet.likewallet", iOSAppId: "1492241404");
        } finally {
          await launch("likewallet://");
          OpenAppstore.launch(
              androidAppId: "likewallet.likewallet", iOSAppId: "1492241404");
//        launch("https://apps.apple.com/us/app/id1504852835");
        }
      } else if (os == "android") {
        try {
          await AppAvailability.launchApp("likewallet.likewallet");
        } catch (e) {
          const url =
              'https://play.google.com/store/apps/details?id=likewallet.likewallet&hl=en';
          if (await canLaunch(url)) {
            await launch(url);
          } else {
            throw 'Could not launch $url';
          }
        }
      } else {}
    }
  }
}
