import 'package:adobe_xd/adobe_xd.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:minilikewallet/provider/home_provider/first_page_provider/first_page_provider.dart';
import 'package:minilikewallet/provider/home_provider/home_provider.dart';
import 'package:minilikewallet/provider/home_provider/index_provider/index_provider.dart';
import 'package:minilikewallet/provider/home_provider/loading_provider/loading_provider.dart';
import 'package:minilikewallet/provider/home_provider/otp_provider/otp_provider.dart';
import 'package:minilikewallet/theme/widget.dart';
import 'package:provider/provider.dart';

class OTPPageComponent {
  OTPPageComponent._();

  static Widget otpBackground() => Container(
        width: 1080.0.w,
        height: 2119.0.h,
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(103.0.h),
          color: const Color(0xff111322),
          // boxShadow: [
          //   BoxShadow(
          //     color: const Color(0x59000000),
          //     offset: Offset(0, 19),
          //     blurRadius: 45,
          //   ),
          // ],
        ),
      );

  static Widget textDidGet() => Text(
        'Didn’t get the code?',
        style: TextStyle(
          fontFamily: 'Proxima Nova',
          fontSize: 35.h,
          color: const Color(0xff1e9fa7),
          letterSpacing: 1.75,
          height: 1.2857142857142858,
        ),
        textHeightBehavior: TextHeightBehavior(applyHeightToFirstAscent: false),
        textAlign: TextAlign.left,
      );

  static Widget buttonResend({String phone, BuildContext context}) {
    LoadingProvider _loadingProvider;
    HomeProvider _homeProvider;
    OTPProvider _otpProvider;
    IndexProvider _indexProvider;
    FirstPageProvider _firstPageProvider;
    _homeProvider = Provider.of<HomeProvider>(context);
    _otpProvider = Provider.of<OTPProvider>(context);
    _loadingProvider = Provider.of<LoadingProvider>(context);
    _indexProvider = Provider.of<IndexProvider>(context);
    _firstPageProvider = Provider.of<FirstPageProvider>(context);

    return SizedBox(
      height: 68.0.h,
      width: 227.0.w,
      child: Stack(
        alignment: Alignment.center,
        children: <Widget>[
          Pinned.fromSize(
            bounds: Rect.fromLTWH(0.0, 0.0, 227.0.w, 68.0.h),
            size: Size(227.0.w, 68.0.h),
            pinLeft: true,
            pinRight: true,
            pinTop: true,
            pinBottom: true,
            child: Container(
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(34.0),
                color: const Color(0xff1e9fa7),
              ),
            ),
          ),
          InkWell(
            onTap: () async {
              _loadingProvider.setStatus(status: true);
              bool login = await _otpProvider.checkLogin();
              _firstPageProvider.setStatus(status: true);
              _loadingProvider.setStatus(status: true);
              if (login) {
                _homeProvider.setMenu(status: true);
                await _indexProvider.loadBalance();
                _loadingProvider.setStatus(status: false);
              } else {
                _homeProvider.setOTP(status: true);
                await _otpProvider.setNumberPhone(phone: phone);
                final res = await _otpProvider.checkExistUser(
                    phoneNumber: _otpProvider.getNumberPhone);
                if (res) {
                  final res = await _otpProvider.sendCodeToPhoneNumber(
                      phoneNumber: _otpProvider.getNumberPhone);
                  if (res) {
                    _loadingProvider.setStatus(status: false);
                  } else {
                    _homeProvider.setOTP(status: false);
                    Navigator.of(context).pop();
                    WidgetTheme.showShortToastRed('เกิดข้อผิดพลาด');
                  }
                } else {
                  Navigator.of(context).pop();
                  WidgetTheme.showShortToastRed('ไม่พบเบอร์ในระบบ');
                }
              }
            },
            child: Text(
              'Resend',
              style: TextStyle(
                fontFamily: 'Proxima Nova',
                fontSize: 33.h,
                color: const Color(0xff000000),
                letterSpacing: 0.99,
              ),
              textAlign: TextAlign.center,
            ),
          ),
        ],
      ),
    );
  }

  static Widget textOTP() => SizedBox(
        width: 9296.0.w,
        child: Text(
          'Enter OTP',
          style: TextStyle(
            fontFamily: 'Proxima Nova',
            fontSize: 60.h,
            color: const Color(0xff08eef3),
            letterSpacing: 1.7999999999999998,
            fontWeight: FontWeight.w300,
          ),
          textAlign: TextAlign.center,
        ),
      );

  static Widget textNumberPhone({String phone}) => SizedBox(
        width: 9296.0.w,
        child: Text(
          phone,
          style: TextStyle(
            fontFamily: 'Proxima Nova',
            fontSize: 45.h,
            color: const Color(0xff08eef3),
            letterSpacing: 1.7999999999999998,
            fontWeight: FontWeight.w300,
          ),
          textAlign: TextAlign.center,
        ),
      );

  static Widget textInputOTP({String numberOTP}) => Column(
        children: [
          Text(
            numberOTP,
            style: TextStyle(
              fontFamily: 'Proxima Nova',
              fontSize: 72.h,
              color: const Color(0xccffffff),
              letterSpacing: 3.6,
              height: 0.7777777777777778,
              shadows: [
                Shadow(
                  color: const Color(0x26000000),
                  offset: Offset(0, 1),
                  blurRadius: 1,
                )
              ],
            ),
            textHeightBehavior:
                TextHeightBehavior(applyHeightToFirstAscent: false),
            textAlign: TextAlign.left,
          ),
          SizedBox(height: 17.h),
          Container(
            height: 1.5.h,
            width: 85.w,
            child: SvgPicture.string(
              '<svg viewBox="0.4 0.4 85.0 1.0" ><path transform="translate(0.45, 0.36)" d="M 0 0 L 85 0" fill="none" stroke="#1e9fa7" stroke-width="1" stroke-miterlimit="4" stroke-linecap="round" /></svg>',
              allowDrawingOutsideViewBox: true,
            ),
          )
        ],
      );
}
