import 'dart:convert';

import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:minilikewallet/config/config.dart';
import 'package:http/http.dart' as http;
import 'package:minilikewallet/library/address_service.dart';
import 'package:minilikewallet/library/configuration_service.dart';
import 'package:minilikewallet/library/crypto.dart';
import 'package:shared_preferences/shared_preferences.dart';

class OTPProvider with ChangeNotifier {
  List<String> textOTP = [];
  String numberPhone = '';
  IConfigurationService configETH;
  CryptoEncryptInterface encrypt;
  IAddressService addressService;
  String keyEncrypt;
  String ivEncrypt;
  OTPProvider();

  List<String> get getTextOTP => textOTP;
  String get getNumberPhone => numberPhone;

  setNumberPhone({String phone}) {
    numberPhone = phone;
    notifyListeners();
  }

  setTextOTP({String number}) {
    textOTP.add(number);
    notifyListeners();
  }

  deleteTextOTP() {
    if (textOTP.length != 0) {
      textOTP.removeLast();
      notifyListeners();
    }
  }

  checkTextOTP() {
    if (textOTP.length == 6)
      return false;
    else if (textOTP.length < 6) return;
    // return textOTP.length <= 5 ? true : false;
  }

  Future<bool> checkExistUser({String phoneNumber}) async {
    String url = Env.apiUrl + '/checkExistUser';
    try {
      var response = await http.post(url, body: {
        'apiKey': Env.APIKEY,
        'secretKey': Env.SECRETKEY,
        'phone_number': phoneNumber
      });
      var body = json.decode(response.body);
      return body['result'];
    } catch (e) {
      return false;
    }
  }

  Future<bool> sendCodeToPhoneNumber({String phoneNumber}) async {
    String url = Env.apiUrl + '/authNoFirebase';
    try {
      var response = await http.post(url, body: {'phone_number': phoneNumber});
      print('Response status: ${response.statusCode}');
      print('Response body: ${response.body}');
      var body = json.decode(response.body);
      if (body['statusCode'] == 200) {
        //success register and sign in
        return true;
      } else {
        return false;
      }
    } catch (e) {
      return false;
    }
  }

  Future<bool> verifyOTP({String phoneNumber, String smsCode}) async {
    print(smsCode);
    String url = Env.apiUrl + '/verifyOTPNoFirebaseNew';
    try {
      var response = await http.post(url,
          body: {'phone_number': phoneNumber, 'codeVerify': smsCode});
      print('Response status: ${response.statusCode}');
      print('Response body: ${response.body}');
      var body = json.decode(response.body);
      if (body['statusCode'] == 200) {
        final customToken = body['token'];

        UserCredential user = await FirebaseAuth.instance
            .signInWithCustomToken(customToken.toString())
            .catchError((error) {
          return false;
        });
        return true;
      } else {
        return false;
      }
    } catch (e) {
      return false;
    }
  }

  Future<bool> setLogin() async {
    encrypt = new CryptoEncrypt();
    final status = await encrypt.generateKey();
    if (status) {
      SharedPreferences pref = await SharedPreferences.getInstance();
      final key = await encrypt.getKeyEncrypt();
      if (key != null) {
        keyEncrypt = key[0];
        ivEncrypt = key[1];
        configETH = new ConfigurationService(pref);
        addressService = new AddressService(configETH);
        pref.setBool('login', false);
        //หลังจากได้ key ทำการเข้าระบบ
      }
    }
    User user = FirebaseAuth.instance.currentUser;

    print(user.phoneNumber);
    print(getTextOTP);
    print(keyEncrypt);
    print(ivEncrypt);

    bool res = await user.getIdTokenResult().then((userDetail) async {
      var url = Env.apiUrl + '/verifyUserAndCreateNoSecret';
      var response = await http.post(url, body: {
        'phone_number': getNumberPhone,
        'password': 'LikeWallet',
        'codeVerify': getTextOTP.join(),
        'register': 'false',
        'keyEncrypt': keyEncrypt,
        'ivEncrypt': ivEncrypt,
        'refCode': 'no',
        'numberID': 'no',
      });

      try {
        var body = json.decode(response.body);
        if (body['statusCode'] == 204 || body['statusCode'] == 200) {
          SharedPreferences pref = await SharedPreferences.getInstance();
          await configETH.setMnemonic(body['seed']);
          pref.setBool('login', true);

          return true;
        } else {
          return false;
        }
      } catch (e) {
        return false;
      }
    });
    return res;
  }

  Future<bool> checkLogin() async {
    SharedPreferences pref = await SharedPreferences.getInstance();
    return pref.get('login') ?? false;
  }
}
