import 'dart:convert';

import 'package:app_settings/app_settings.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:contacts_service/contacts_service.dart';
import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:http/http.dart' as http;
import 'package:flutter/material.dart';
import 'package:minilikewallet/config/config.dart';
import 'package:minilikewallet/library/auth.dart';
import 'package:minilikewallet/library/configuration_service.dart';
import 'package:minilikewallet/library/ethcontract.dart';
import 'package:minilikewallet/middleware/check_auth.dart';
import 'package:minilikewallet/middleware/get_mnemonic.dart';
import 'package:minilikewallet/model/contact.dart';
import 'package:minilikewallet/model/list.dart';
import 'package:permission_handler/permission_handler.dart';

class SendProvider with ChangeNotifier {
  List<String> sendAmount = [];
  TextEditingController noteController = TextEditingController();
  String nameTo = '';
  String phone = '';
  String addressTo = '';
  String nameFOMR = '';
  String textContact = '';
  bool loading = false;
  bool loadingConfirm = false;
  bool loadingContact = false;
  bool loadingContactForeign = false;
  bool loadingStore = false;
  bool tapAddNote = false;
  bool confirm = false;
  bool contract = false;
  bool store = false;
  List<shop> listShop = [];
  PermissionStatus permissionStatus;
  double rateCurrency = 0.0;
  List<Contacts> listContact = [];
  List<Contacts> listContactForeign = [];
  Iterable<Contact> contacts;
  SendProvider();

  List<String> get getAmount => sendAmount;
  bool get getTapAddNote => tapAddNote;
  bool get getConfirm => confirm;
  bool get getContract => contract;
  bool get getStore => store;
  PermissionStatus get getPermissionStatus => permissionStatus;
  String get getNameTo => nameTo;
  String get getNameFOMR => nameFOMR;
  String get getPhone => phone;
  bool get getLoading => loading;
  bool get getLoadingConfirm => loadingConfirm;
  bool get getLoadingContact => loadingContact;
  bool get getLoadingContactForeign => loadingContactForeign;
  bool get getLoadingStore => loadingStore;
  String get getAddressTo => addressTo;
  double get getRateCurrency => rateCurrency;
  List<Contacts> get getListContact => listContact;

  BaseAuth auth;
  Dio dio = new Dio();

  setLoading({bool status}) {
    loading = status;
    notifyListeners();
  }

  setLoadingConfirm({bool status}) {
    loadingConfirm = status;
    notifyListeners();
  }

  setLoadingContact({bool status}) {
    loadingContact = status;
    notifyListeners();
  }

  changeAmount({var value}) {
    sendAmount += value;
    notifyListeners();
  }

  deleteAmount() {
    if (sendAmount.length >= 1) {
      sendAmount.removeLast();
    }
    notifyListeners();
  }

  setTextAmount({var number}) {
    sendAmount.add(number.toString());
    notifyListeners();
  }

  changeAddNote({bool status}) {
    tapAddNote = status;
    notifyListeners();
  }

  changeConfirm({bool status}) {
    confirm = status;
    notifyListeners();
  }

  changeStore({bool status}) {
    store = status;
    notifyListeners();
  }

  changeContact({bool status}) {
    contract = status;
    notifyListeners();
  }

  setListContact({List<Contacts> data}) {
    listContact = data;
    notifyListeners();
  }

  void getNameTO({String address}) async {
    auth = Auth();
    addressTo = address;
    loading = true;
    final _token = await auth.getTokenFirebase();
    try {
      Response response = await dio.post(Env.apiUrl + "/searchNameNew",
          data: {"term": address, "_token": _token});
      if (response.data["statusCode"] == 200) {
        print(response.data["result"]['name']);
        print(response.data["result"]);
        nameTo = response.data["result"]['name'];
        phone = response.data["result"]['phoneNumber'];
        loading = false;
        notifyListeners();
      }
    } catch (e) {
      print(e);
    }
  }

  void getNameFROM({String address}) async {
    auth = Auth();
    final _token = await auth.getTokenFirebase();
    try {
      Response response = await dio.post(Env.apiUrl + "/searchNameNew",
          data: {"term": address, "_token": _token});
      if (response.data["statusCode"] == 200) {
        print(response.data["result"]['name']);
        print(response.data["result"]);
        nameFOMR = response.data["result"]['name'];
        notifyListeners();
      }
    } catch (e) {
      print(e);
    }
  }

  Future<bool> statusPermission() async {
    var status = await Permission.photos.status;
    final permissionStatus = await Permission.photos.request();
    if (permissionStatus == PermissionStatus.granted) {
      return true;
    } else if (status.isDenied ||
        status.isUndetermined ||
        status.isPermanentlyDenied) {
      return false;
    }
  }

  getRateCurrencySymbol({String symbol}) {
    FirebaseFirestore.instance
        .collection('exchangeFiat')
        .doc(symbol == 'none' ? "THB-LIKE" : "THB-" + symbol)
        .get()
        .then((DocumentSnapshot ds) {
      print(ds.data()["rate"]);
      if (ds.data()["to"] == 'LIKE') {
        rateCurrency = ds.data()["rate"];
      } else {
        rateCurrency = 1 / ds.data()["rate"];
        print(rateCurrency);
      }
    });
    notifyListeners();
  }

  Future<String> signTransaction({
    IConfigurationService configETH,
    EthContract eth,
    CurCheckAuth login,
    GetMnemonic seed,
    String addressTo,
    String amount,
    String contract,
    String callFunction,
    String abi,
  }) async {
    double rate = 100;
    String tx = '';
    num avaiBalance = await eth.getBalance(address: configETH.getAddress());
    if ((avaiBalance >= double.parse(amount) * rateCurrency * rate)) {
      //เช็ต login
      bool loginStatus = await login.checkLogin();
      if (loginStatus) {
        //เช็ต Seed กระเป๋า
        String value = await seed.getMnemonic();
        if (value.isNotEmpty) {
          // final mnemonic = value.split(":")[0];
          final pkEth = value.split(":")[1];

//        eth.sendTransaction(pk: pketh, to: nameTO.trim(), value: (double.parse(amount)*rateCurrency*rate).toString()).then((tx) {
          if (contract == 'no') {
            final resultTX = await eth.transferMessage(
                pk: pkEth,
                to: addressTo.trim(),
                value: (double.parse(amount) * rateCurrency * rate).toString(),
                message: noteController.text.toString());

            tx = resultTX;
            print('tx : $tx');
          } else if (contract.length == 42 &&
              callFunction != 'no' &&
              abi != 'no') {
            final resultTX = await eth.dynamicSendContract(
                pk: pkEth,
                to: addressTo.trim(),
                value: (double.parse(amount) * rateCurrency * rate).toString(),
                message: noteController.text.toString(),
                callFunction: callFunction,
                abi: abi,
                contractAddress: contract);
            tx = resultTX;
            print('tx : $tx');
          }
        }
      }
    } else {
      print('คุณมี LIKE ไม่เพียงพอ');
      // showShortToast(AppLocalizations.of(context).translate('not_enough') +
      //     AppLocalizations.of(context).translate('not_enough_tail') +
      //     " " +
      //     (avaiBalance).toString() +
      //     "LIKE");
    }
    return tx;
  }

  Future<PermissionStatus> permissionContact() async {
    final PermissionStatus permission = await Permission.contacts.status;
    if (permission != PermissionStatus.granted &&
        permission != PermissionStatus.denied) {
      final Map<Permission, PermissionStatus> permissionStatus =
          await [Permission.contacts].request();
      return permissionStatus[Permission.contacts] ??
          PermissionStatus.undetermined;
    } else {
      return permission;
    }
  }

  Future<List<Contacts>> checkCodePhone(
      {String phone, BuildContext context}) async {
    if (phone.substring(0, 3) == '+66') {
      print('Thailand');
      await getContactsLocal(context: context, codePhone: '+66');
      // await getContactsForeign(
      //     context: context, codePhone1: '+855', codePhone2: '+856');
    } else if (phone.substring(0, 4) == '+855') {
      print('Cambodia');
      await getContactsLocal(context: context, codePhone: '+855');
    } else if (phone.substring(0, 4) == '+856') {
      print('Laos');
      await getContactsLocal(context: context, codePhone: '+856');
    }
  }

  List<List<T>> _generateChunks<T>(List<T> inList, int chunkSize) {
    List<List<T>> outList = [];
    List<T> tmpList = [];
    int counter = 0;

    for (int current = 0; current < inList.length; current++) {
      if (counter != chunkSize) {
        tmpList.add(inList[current]);
        counter++;
      }
      if (counter == chunkSize || current == inList.length - 1) {
        outList.add(tmpList.toList());
        tmpList.clear();
        counter = 0;
      }
    }

    return outList;
  }

  Future<List<Contacts>> getContactsLocal(
      {BuildContext context, String codePhone}) async {
    setLoadingContact(status: true);
    textContact = '';
    List phoneContactTH = [];
    final PermissionStatus permissionStatus = await permissionContact();

    if (permissionStatus == PermissionStatus.granted) {
      contacts = await ContactsService.getContacts();
      // print(contacts);
      contacts.forEach((element) async {
        element.phones.forEach((phone) async {
          // print(phone.value.toString());
          if (phone.value.toString().substring(0, 1) == '0') {
            final p = phone.value.toString();
            phoneContactTH
                .add(p.replaceFirst('0', codePhone).replaceAll(' ', ''));
          } else {}
        });
      });

      print("รายชื่อ :" + phoneContactTH.length.toString());
      int chunkSize = 10;
      List<List<dynamic>> chunks = _generateChunks(phoneContactTH, chunkSize);
      print(chunks);
      for (int i = 0; i < chunks.length; i++) {
        QuerySnapshot th = await FirebaseFirestore.instance
            .collection('addressDNS')
            .where('phoneNumber', whereIn: chunks[i])
            .get();
        if (th.docs.isNotEmpty) {
          listContact.add(Contacts.fromJson(th.docs[0].data()));
          loadingContact = false;
          notifyListeners();
        } else {}
      }

      if (listContact.length == 0) {
        print('listContact' + listContact.length.toString());
        loadingContact = false;
        textContact = 'ไม่พบรายชื่อ';
        notifyListeners();
      }
    } else if (permissionStatus == PermissionStatus.undetermined ||
        permissionStatus == PermissionStatus.denied) {
      await Permission.contacts.request();
      // getContacts(context: context);
    } else if (permissionStatus == PermissionStatus.permanentlyDenied) {
      //If permissions have been denied show standard cupertino alert dialog
      showDialog(
          context: context,
          builder: (BuildContext context) => CupertinoAlertDialog(
                title: Text('ข้อผิดพลาดในการอนุญาต'),
                content: Text('โปรดเปิดใช้งานการเข้าถึงผู้ติดต่อ '
                    'การอนุญาตในการตั้งค่าระบบ'),
                actions: <Widget>[
                  CupertinoDialogAction(
                      child: Text('ตกลง'),
                      onPressed: () async {
                        AppSettings.openAppSettings().then((value) {
                          Navigator.of(context).pop();
                          loadingContact = false;
                          notifyListeners();
                        });
                      })
                ],
              ));
    }
    return listContact;
  }

  Future<List<Contacts>> getContactsForeign(
      {BuildContext context, String codePhone1, String codePhone2}) async {
    setLoadingContact(status: true);
    textContact = '';
    List phoneContact = [];

    final PermissionStatus permissionStatus = await permissionContact();

    if (permissionStatus == PermissionStatus.granted) {
      contacts = await ContactsService.getContacts();
      contacts.forEach((element) async {
        element.phones.forEach((phone) async {
          if (phone.value.toString().substring(0, 1) == '0') {
            final p = phone.value.toString();
            phoneContact
                .add(p.replaceFirst('0', codePhone1).replaceAll(' ', ''));
            phoneContact
                .add(p.replaceFirst('0', codePhone2).replaceAll(' ', ''));
          } else {}
        });
      });

      print("รายชื่อ :" + phoneContact.length.toString());
      int chunkSize = 10;
      List<List<dynamic>> chunks = _generateChunks(phoneContact, chunkSize);
      print(chunks);
      for (int i = 0; i < chunks.length; i++) {
        QuerySnapshot th = await FirebaseFirestore.instance
            .collection('addressDNS')
            .where('phoneNumber', whereIn: chunks[i])
            .get();
        if (th.docs.isNotEmpty) {
          listContactForeign.add(Contacts.fromJson(th.docs[0].data()));
          loadingContact = false;
          notifyListeners();
        } else {}
      }

      if (listContactForeign.length == 0) {
        print('listContactForeign' + listContactForeign.length.toString());
        loadingContact = false;
        textContact = 'ไม่พบรายชื่อ';
        notifyListeners();
      }
    } else if (permissionStatus == PermissionStatus.undetermined ||
        permissionStatus == PermissionStatus.denied) {
      await Permission.contacts.request();
      // getContacts(context: context);
    } else if (permissionStatus == PermissionStatus.permanentlyDenied) {
      //If permissions have been denied show standard cupertino alert dialog
      showDialog(
          context: context,
          builder: (BuildContext context) => CupertinoAlertDialog(
                title: Text('ข้อผิดพลาดในการอนุญาต'),
                content: Text('โปรดเปิดใช้งานการเข้าถึงผู้ติดต่อ '
                    'การอนุญาตในการตั้งค่าระบบ'),
                actions: <Widget>[
                  CupertinoDialogAction(
                      child: Text('ตกลง'),
                      onPressed: () async {
                        AppSettings.openAppSettings().then((value) {
                          Navigator.of(context).pop();
                          loadingContact = false;
                          notifyListeners();
                        });
                      })
                ],
              ));
    }
    return listContact;
  }

  void getShop() async {
    //โหลดรอข้อมูล
    listShop = [];
    loadingStore = true;
    var response = await http.post(Env.apiUrl + '/listSpendkpayShop', body: {
      "apiKey": Env.APIKEY,
      "secretKey": Env.SECRETKEY,
    });
    if (response.statusCode == 200) {
      loadingStore = false;
      var body = json.decode(response.body);
      print(response.body);
      for (Map user in body["result"]) {
        listShop.add(shop.fromJson(user));
        notifyListeners();
      }
    } else {
      loadingStore = false;
    }

    // return [_list, group];
  }
}
