import 'package:adobe_xd/adobe_xd.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:minilikewallet/config/config.dart';
import 'package:minilikewallet/library/auth.dart';
import 'package:minilikewallet/provider/home_provider/send_provider/send_provider.dart';
import 'package:minilikewallet/scan_qrcode/scan_qrcode.dart';
import 'package:minilikewallet/theme/image.dart';
import 'package:minilikewallet/theme/widget.dart';
import 'package:pattern_formatter/numeric_formatter.dart';
import 'dart:io';
import 'package:intl/intl.dart' as formatIntl;
import 'package:provider/provider.dart';

class SendConfirmComponent {
  SendConfirmComponent._();
  BaseAuth auth;

  static Widget sendBackground() => SizedBox(
        width: 1080.0.w,
        height: 1064.1.h,
        child: Stack(
          children: <Widget>[
            Pinned.fromSize(
              bounds: Rect.fromLTWH(0.0, 0.0, 1080.0.w, 627.0.h),
              size: Size(1080.0.w, 1064.1.h),
              pinLeft: true,
              pinRight: true,
              pinTop: true,
              fixedHeight: true,
              child: Container(
                decoration: BoxDecoration(
                  gradient: LinearGradient(
                    begin: Alignment(-1.0, 0.0),
                    end: Alignment(1.0, 0.0),
                    colors: [const Color(0xff00ebf0), const Color(0xff00b5c4)],
                    stops: [0.0, 1.0],
                  ),
                ),
              ),
            ),
            Pinned.fromSize(
              bounds: Rect.fromLTWH(0.0, 573.4.h, 1080.0.w, 490.7.h),
              size: Size(1080.0.w, 1064.1.h),
              pinLeft: true,
              pinRight: true,
              pinBottom: true,
              fixedHeight: true,
              child: SvgPicture.string(
                '<svg viewBox="0.0 573.4 1080.0 490.7" ><defs><filter id="shadow"><feDropShadow dx="0" dy="19" stdDeviation="45"/></filter></defs><path transform="translate(0.0, -273.85)" d="M 1080 847.29443359375 L 1080 1198 C 1080 1275.31982421875 1017.319885253906 1338 940 1338 L 140 1338 C 62.68013000488281 1338 0 1275.31982421875 0 1198 L 0 847.29443359375 L 1080 847.29443359375 Z" fill="#16192f" stroke="none" stroke-width="1" stroke-miterlimit="4" stroke-linecap="butt" filter="url(#shadow)"/></svg>',
                allowDrawingOutsideViewBox: true,
                fit: BoxFit.fill,
              ),
            ),
            Pinned.fromSize(
              bounds: Rect.fromLTWH(40.0.w, 251.7.h, 1000.0.w, 640.5.h),
              size: Size(1080.0.w, 1064.1.h),
              pinLeft: true,
              pinRight: true,
              fixedHeight: true,
              child: SvgPicture.string(
                '<svg viewBox="40.0 251.7 1000.0 640.5" ><defs><linearGradient id="gradient" x1="0.5" y1="0.0" x2="0.5" y2="1.0"><stop offset="0.0" stop-color="#ff232e4a"  /><stop offset="1.0" stop-color="#ff16192f"  /></linearGradient></defs><path transform="translate(40.0, 251.68)" d="M 78.66274261474609 0 L 921.3373413085938 0 C 964.7815551757813 0 999.9999389648438 32.79358673095703 999.9999389648438 73.24652862548828 L 999.9999389648438 567.2597045898438 C 999.9999389648438 607.7127685546875 964.7815551757813 640.50634765625 921.3373413085938 640.50634765625 L 78.66274261474609 640.50634765625 C 35.21850967407227 640.50634765625 0 607.7127685546875 0 567.2597045898438 L 0 73.24652862548828 C 0 32.79358673095703 35.21850967407227 0 78.66274261474609 0 Z" fill="url(#gradient)" stroke="none" stroke-width="1" stroke-miterlimit="4" stroke-linecap="butt" /></svg>',
                allowDrawingOutsideViewBox: true,
                fit: BoxFit.fill,
              ),
            ),
          ],
        ),
      );

  static Widget sendTotal({String amount}) {
    final f = new formatIntl.NumberFormat("###,###.##");
    return Padding(
      padding: EdgeInsets.symmetric(horizontal: 150.w),
      child: Row(
        // mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text(
            'Send',
            style: TextStyle(
              fontFamily: 'Proxima Nova',
              fontSize: 33.h,
              color: const Color(0x99ffffff),
              letterSpacing: 3.3000000000000003,
              height: 1.7272727272727273,
            ),
            textHeightBehavior:
                TextHeightBehavior(applyHeightToFirstAscent: false),
            textAlign: TextAlign.left,
          ),
          SizedBox(width: 67.w),
          Expanded(
            child: Text(
              f.format(double.parse(amount)).toString() ?? '',
              style: TextStyle(
                fontFamily: 'Proxima Nova',
                fontSize: 39.h,
                color: const Color(0xff9c77ff),
                letterSpacing: 1.17,
                height: 1.4615384615384615,
              ),
              maxLines: 1,
              overflow: TextOverflow.ellipsis,
              textHeightBehavior:
                  TextHeightBehavior(applyHeightToFirstAscent: false),
              textAlign: TextAlign.left,
            ),
          ),
          Text(
            'Likepoint',
            style: TextStyle(
              fontFamily: 'Proxima Nova',
              fontSize: 30.h,
              color: const Color(0x99ffffff),
              letterSpacing: 3,
              height: 1.9,
            ),
            maxLines: 1,
            overflow: TextOverflow.ellipsis,
            textHeightBehavior:
                TextHeightBehavior(applyHeightToFirstAscent: false),
            textAlign: TextAlign.right,
          ),
        ],
      ),
    );
  }

  static Widget sendTo({BuildContext context}) {
    TextEditingController phoneController = TextEditingController();
    SendProvider _sendProvider;
    _sendProvider = Provider.of<SendProvider>(context, listen: true);
    return Padding(
      padding: EdgeInsets.symmetric(horizontal: 150.w),
      child: Row(
        // mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text(
            'To',
            style: TextStyle(
              fontFamily: 'Proxima Nova',
              fontSize: 33.h,
              color: const Color(0x99ffffff),
              letterSpacing: 3.3000000000000003,
              height: 1.7272727272727273,
            ),
            textHeightBehavior:
                TextHeightBehavior(applyHeightToFirstAscent: false),
            textAlign: TextAlign.left,
          ),
          SizedBox(width: 113.w),
          // _sendProvider.getNameTo == ''
          //     // ? Expanded(
          //     //     child: TextFormField(
          //     //     maxLines: 1,
          //     //     maxLength: 13,
          //     //     onTap: () {
          //     //       // tap = true;
          //     //     },
          //     //     controller: phoneController,
          //     //     textAlign: TextAlign.left,
          //     //     textInputAction: TextInputAction.next,
          //     //     keyboardType: TextInputType.text,
          //     //     textAlignVertical: TextAlignVertical.top,
          //     //     cursorColor: Colors.grey,
          //     //     readOnly: false,
          //     //     autofocus: true,
          //     //     onChanged: (value) {},
          //     //     onFieldSubmitted: (value) {
          //     //       phoneController.text = value;
          //     //     },
          //     //     style: TextStyle(
          //     //         color: Colors.white,
          //     //         letterSpacing: 0.1,
          //     //         fontFamily: '',
          //     //         fontWeight: FontWeight.w300,
          //     //         fontSize: 36.h),
          //     //     decoration: InputDecoration(
          //     //         border: InputBorder.none,
          //     //         contentPadding: EdgeInsets.only(bottom: 30.h),
          //     //         hintText: 'Number phone',
          //     //         hintStyle: TextStyle(
          //     //             color: Colors.grey,
          //     //             letterSpacing: 0.1,
          //     //             fontFamily: '',
          //     //             fontWeight: FontWeight.w300,
          //     //             fontSize: 36.h)),
          //     //   ))
          //     // :
          Expanded(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text(
                  _sendProvider.getNameTo,
                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(
                    fontFamily: 'Proxima Nova',
                    fontSize: 39.h,
                    color: const Color(0xff9c77ff),
                    letterSpacing: 1.17,
                    height: 1.4615384615384615,
                  ),
                  textHeightBehavior:
                      TextHeightBehavior(applyHeightToFirstAscent: false),
                  textAlign: TextAlign.left,
                ),
                Text(
                  _sendProvider.getPhone,
                  style: TextStyle(
                    fontFamily: 'Proxima Nova',
                    fontSize: 39.h,
                    color: Colors.white,
                    letterSpacing: 1.17,
                    height: 1.4615384615384615,
                  ),
                  maxLines: 1,
                  overflow: TextOverflow.ellipsis,
                  textHeightBehavior:
                      TextHeightBehavior(applyHeightToFirstAscent: false),
                  textAlign: TextAlign.left,
                ),
              ],
            ),
          ),
          InkWell(
            onTap: () async {
              final address = await Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (context) => ScanQrCode(),
                ),
              );
              if (address == null) {
                print('ไม่ได้รับ address');
              } else {
                _sendProvider.getNameTO(address: address);
              }
            },
            child: Image.asset(
              ImageTheme.iconQrCode,
              height: 59.44.h,
              package: 'minilikewallet',
            ),
          ),
          SizedBox(width: 30.w),
          InkWell(
              onTap: () async {
                if (_sendProvider.getStore == false) {
                  _sendProvider.getShop();
                  _sendProvider.nameTo = '';
                  _sendProvider.phone = '';
                  _sendProvider.setLoadingContact(status: false);
                  _sendProvider.changeContact(status: false);
                  _sendProvider.changeConfirm(status: false);
                  _sendProvider.changeStore(status: true);
                } else {
                  _sendProvider.changeStore(status: false);
                }
              },
              child: Icon(
                _sendProvider.getStore
                    ? Icons.local_grocery_store_outlined
                    : Icons.local_grocery_store,
                color: Colors.white,
              )),
        ],
      ),
    );
  }

  static Widget addNote() => InkWell(
        child: Padding(
          padding: EdgeInsets.only(left: 308.w),
          child: Row(
            children: [
              SizedBox(
                width: 174.0.w,
                height: 60.0.h,
                child: Stack(
                  alignment: Alignment.center,
                  children: <Widget>[
                    Pinned.fromSize(
                      bounds: Rect.fromLTWH(0.0, 0.0, 174.0.w, 60.0.h),
                      size: Size(174.0.w, 60.0.h),
                      pinLeft: true,
                      pinRight: true,
                      pinTop: true,
                      pinBottom: true,
                      child: Stack(
                        children: <Widget>[
                          Pinned.fromSize(
                            bounds: Rect.fromLTWH(0.0, 0.0, 174.0.w, 60.0.h),
                            size: Size(174.0.w, 60.0.h),
                            pinLeft: true,
                            pinRight: true,
                            pinTop: true,
                            pinBottom: true,
                            child: Container(
                              decoration: BoxDecoration(
                                borderRadius: BorderRadius.circular(40.0),
                                color: const Color(0xffa989ff),
                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                    Text(
                      'Add Note',
                      style: TextStyle(
                        fontFamily: 'Proxima Nova',
                        fontSize: 24.h,
                        color: const Color(0xff000000),
                        letterSpacing: 1.2000000000000002,
                      ),
                      textAlign: TextAlign.center,
                    ),
                  ],
                ),
              ),
            ],
          ),
        ),
      );
  static Widget swiftContacts() => InkWell(
        child: Padding(
          padding: EdgeInsets.only(left: 308.w),
          child: Row(
            children: [
              Container(
                width: 323.0.w,
                height: 71.0.h,
                alignment: Alignment.center,
                decoration: new BoxDecoration(
                    color: Color(0xff848383),
                    borderRadius: new BorderRadius.all(Radius.circular(40.0))),
                child: Text(
                  'เพื่อนต่างประเทศ',
                  style: TextStyle(
                    fontFamily: 'Proxima Nova',
                    fontSize: 30.h,
                    color: Colors.white,
                    letterSpacing: 1.2000000000000002,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
            ],
          ),
        ),
      );

  static Widget note({TextEditingController noteController}) => Padding(
        padding: EdgeInsets.symmetric(horizontal: 173.w),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Text(
              'Note',
              style: TextStyle(
                fontFamily: 'Proxima Nova',
                fontSize: 33.h,
                color: const Color(0x99ffffff),
                letterSpacing: 3.3000000000000003,
                height: 1.7272727272727273,
              ),
              textHeightBehavior:
                  TextHeightBehavior(applyHeightToFirstAscent: false),
              textAlign: TextAlign.left,
            ),
            SizedBox(width: 68.w),
            Expanded(
                child: TextFormField(
              maxLines: 3,
              onTap: () {
                // tap = true;
              },
              controller: noteController,
              textAlign: TextAlign.left,
              textInputAction: TextInputAction.next,
              keyboardType: TextInputType.text,
              textAlignVertical: TextAlignVertical.top,
              cursorColor: Colors.grey,
              readOnly: false,
              autofocus: true,
              onChanged: (value) {},
              style: TextStyle(
                  color: Colors.white,
                  letterSpacing: 0.1,
                  fontFamily: '',
                  fontWeight: FontWeight.w300,
                  fontSize: 36.h),
              decoration: InputDecoration(
                  border: InputBorder.none,
                  contentPadding: EdgeInsets.only(bottom: 30.h),
                  hintText: '',
                  hintStyle: TextStyle(
                      color: Colors.grey,
                      letterSpacing: 0.1,
                      fontFamily: '',
                      fontWeight: FontWeight.w300,
                      fontSize: 36.h)),
            )),
          ],
        ),
      );
  static Widget backButton() => Container(
        padding: EdgeInsets.symmetric(horizontal: 173.w, vertical: 30.h),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            WidgetTheme.backButton(),
          ],
        ),
      );

  static Widget buttonConfirm() => SizedBox(
        width: 798.0.w,
        height: 118.0.h,
        child: Stack(
          alignment: Alignment.center,
          children: <Widget>[
            Pinned.fromSize(
              bounds: Rect.fromLTWH(0.0, 0.0, 798.0.w, 118.0.h),
              size: Size(798.0.w, 118.0.h),
              pinLeft: true,
              pinRight: true,
              pinTop: true,
              pinBottom: true,
              child: Container(
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(59.0),
                  gradient: LinearGradient(
                    begin: Alignment(-1.0, 0.09),
                    end: Alignment(1.0, 0.1),
                    colors: [
                      const Color(0xff9667fb),
                      const Color(0xff7b75fd),
                      const Color(0xff5648fa)
                    ],
                    stops: [0.0, 0.547, 1.0],
                  ),
                  boxShadow: [
                    BoxShadow(
                      color: const Color(0x29000000),
                      offset: Offset(0, 9),
                      blurRadius: 24,
                    ),
                  ],
                ),
              ),
            ),
            Text(
              'Confirm',
              style: TextStyle(
                fontFamily: 'Proxima Nova',
                fontSize: 39.h,
                color: const Color(0xffffffff),
                letterSpacing: 1.9500000000000002,
                fontWeight: FontWeight.w600,
              ),
              textAlign: TextAlign.center,
            ),
          ],
        ),
      );
  static Widget buttonCancel() => SizedBox(
        width: 792.0.w,
        height: 123.0.h,
        child: Stack(
          alignment: Alignment.center,
          children: <Widget>[
            Text(
              'Cancel',
              style: TextStyle(
                fontFamily: 'Proxima Nova',
                fontSize: 39.h,
                color: const Color(0xff000000),
                letterSpacing: 1.9500000000000002,
                fontWeight: FontWeight.w600,
              ),
              textAlign: TextAlign.center,
            ),
            Pinned.fromSize(
              bounds: Rect.fromLTWH(0.0, 0.0, 792.0.w, 123.0.h),
              size: Size(792.0.w, 123.0.h),
              pinLeft: true,
              pinRight: true,
              pinTop: true,
              pinBottom: true,
              child: Container(
                decoration: BoxDecoration(),
              ),
            ),
          ],
        ),
      );
}
