import 'package:adobe_xd/adobe_xd.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'dart:ui' as ui;
import 'package:intl/intl.dart' as formatIntl;

import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:minilikewallet/home/button_action/button_action.dart';
import 'package:minilikewallet/provider/home_provider/home_provider.dart';
import 'package:minilikewallet/provider/home_provider/index_provider/index_provider.dart';
import 'package:minilikewallet/provider/home_provider/send_provider/send_provider.dart';
import 'package:minilikewallet/send/component/send_component.dart';
import 'package:minilikewallet/theme/color.dart';
import 'package:minilikewallet/theme/image.dart';
import 'package:minilikewallet/theme/widget.dart';
import 'package:provider/provider.dart';

class SendScreen extends StatefulWidget {
  @override
  _SendScreen createState() => _SendScreen();
}

class _SendScreen extends State<SendScreen> {
  @override
  void initState() {
    // TODO: implement initState
    super.initState();
  }

  HomeProvider _getLikeWallet;
  IndexProvider _indexProvider;
  SendProvider _sendProvider;
  final f = new formatIntl.NumberFormat("###,###.##");

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();
    _getLikeWallet = Provider.of<HomeProvider>(context);
    _indexProvider = Provider.of<IndexProvider>(context);
    _sendProvider = Provider.of<SendProvider>(context, listen: false);
  }

  @override
  Widget build(BuildContext context) {
    ScreenUtil.init(context,
        designSize: Size(1080, 2340), allowFontScaling: false);
    return Container(
      color: Colors.transparent,
      height: MediaQuery.of(context).size.height - 390.h,
      child: Stack(
        children: [
          //-----------------พื้นหลัง------------------//
          Positioned(
            top: -174.h,
            child: SendComponent.sendBackground(),
          ),
          Container(
            color: Colors.transparent,
            height: MediaQuery.of(context).size.height - 390.h,
            width: MediaQuery.of(context).size.width,
            child: Column(
              children: [
                SizedBox(height: 166.h),
                //------------ หัวข้อ ---------------//
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Container(
                      width: MediaQuery.of(context).size.width / 3,
                      child: FlatButton(
                        onPressed: () {
                          _getLikeWallet.setSend(status: false);
                          _getLikeWallet.setMenu(status: true);
                          _sendProvider.sendAmount = [];
                        },
                        child: WidgetTheme.backButton(),
                      ),
                    ),
                    Container(
                      width: MediaQuery.of(context).size.width / 3,
                      alignment: Alignment.center,
                      child: SendComponent.sendTitle(),
                    ),
                    Container(
                      width: MediaQuery.of(context).size.width / 3,
                    ),
                  ],
                ),
                SizedBox(height: 227.6.h),
                //------------ INPUT  ---------------//
                textFiled(),
                SizedBox(height: 199.h),
                //------------ ปุ่ม NEXT  ---------------//
                InkWell(
                    onTap: () {
                      if (_sendProvider.getAmount.length > 0) {
                        double value = double.parse(_sendProvider.sendAmount
                            .join()
                            .replaceAll(',', ''));
                        if (value > 0) {
                          if (value <= _indexProvider.getBalance) {
                            _getLikeWallet.setSendConfirm(status: true);
                            _getLikeWallet.setSend(status: false);
                            _getLikeWallet.setMenu(status: false);
                          } else {
                            WidgetTheme.showShortToastRed('ยอดไม่เพียงพอ');
                          }
                        } else {
                          WidgetTheme.showShortToastRed('ยอดต้องมากกว่า 0');
                        }
                      } else {
                        WidgetTheme.showShortToastRed('กรอกจำนวน');
                      }
                      // _getLikeWallet.setSendConfirm(status: true);
                      // _getLikeWallet.setSend(status: false);
                      // _getLikeWallet.setMenu(status: false);
                    },
                    child: SendComponent.next(
                        sendAmount: _sendProvider.getAmount.join() ?? '')),
                //------------ numberPad  --------ป-------//
                SendComponent.numberPad(context: context),
                SendComponent.bottomNumberPad(context: context),
                Expanded(child: Container()),
                InkWell(
                    onTap: () async {
                      _getLikeWallet.setMenu(status: true);
                      _getLikeWallet.setSend(status: false);
                    },
                    child: Container(
                        height: 30.h,
                        width: 60.w,
                        margin: EdgeInsets.all(20.w),
                        padding: EdgeInsets.symmetric(vertical: 10.h),
                        child: ButtonAction.closeMenuWhite())),
                SizedBox(height: 34.9.h),
              ],
            ),
          ),
        ],
      ),
    );
  }

  Widget textFiled() => Container(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Expanded(
              child: Image.asset(
                ImageTheme.iconLike,
                height: 84.37.h,
                package: 'minilikewallet',
              ),
            ),
            Expanded(
              child: Column(
                children: [
                  SendComponent.sendFieldAmount(
                      text: _sendProvider.getAmount.join()),
                  SendComponent.sendCurrentAmount(
                      text: _sendProvider.getAmount.join(), context: context),
                ],
              ),
            ),
            Expanded(child: Container())
          ],
        ),
      );

  Widget bottomNumberPad() => Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Container(
            margin: EdgeInsets.symmetric(horizontal: 110.w, vertical: 58.w),
            child: SvgPicture.string(
              '<svg viewBox="271.9 823.3 14.3 14.3" ><path transform="translate(273.78, 826.67)" d="M -1.835999488830566 3.788349628448486 C -1.835999488830566 1.180222749710083 -1.137394428253174 -0.6672008633613586 0.2598160803318024 -1.753920197486877 C 1.657027244567871 -2.840639591217041 3.318154811859131 -3.384000301361084 5.24320125579834 -3.384000301361084 C 7.168247699737549 -3.384000301361084 8.844900131225586 -2.840639591217041 10.27316188812256 -1.753920197486877 C 11.70142269134521 -0.6672008633613586 12.41555213928223 1.180222749710083 12.41555213928223 3.788349628448486 C 12.41555213928223 6.334379196166992 11.70142269134521 8.166276931762695 10.27316188812256 9.28404426574707 C 8.844900131225586 10.40181255340576 7.168247699737549 10.96069812774658 5.24320125579834 10.96069812774658 C 3.318154811859131 10.96069812774658 1.657027244567871 10.40181255340576 0.2598160803318024 9.28404426574707 C -1.137394428253174 8.166276931762695 -1.835999488830566 6.334379196166992 -1.835999488830566 3.788349628448486 Z" fill="#08eef3" stroke="none" stroke-width="1" stroke-miterlimit="4" stroke-linecap="butt" /></svg>',
              allowDrawingOutsideViewBox: true,
              height: 14.34.h,
            ),
          ),
          Container(
            margin: EdgeInsets.symmetric(horizontal: 110.w, vertical: 58.w),
            child: SizedBox(
              width: 53.0.w,
              child: Text(
                '0',
                style: TextStyle(
                  fontFamily: 'Noto Sans',
                  fontSize: 50.h,
                  color: const Color(0xff08eef3),
                ),
                textAlign: TextAlign.center,
              ),
            ),
          ),
          InkWell(
            onTap: () {
              _sendProvider.deleteAmount();
            },
            child: Container(
              margin: EdgeInsets.symmetric(horizontal: 110.w, vertical: 58.w),
              child: SizedBox(
                width: 52.0.w,
                height: 41.0.h,
                child: Stack(
                  children: <Widget>[
                    SvgPicture.string(
                      '<svg viewBox="0.0 0.0 51.6 41.1" ><path transform="translate(-647.33, -246.92)" d="M 698.93701171875 257.8011779785156 C 698.9277954101563 256.8959655761719 698.849609375 255.9954376220703 698.7067260742188 255.1040191650391 C 698.5408935546875 254.1205902099609 698.2298583984375 253.1692962646484 697.785400390625 252.2802581787109 C 697.329345703125 251.3843231201172 696.7329711914063 250.5666656494141 696.0234375 249.8572235107422 C 695.3140869140625 249.1455535888672 694.4964599609375 248.5536346435547 693.6027221679688 248.0998382568359 C 692.7136840820313 247.6484222412109 691.7623901367188 247.3397674560547 690.7811279296875 247.1785736083984 C 689.873779296875 247.0219268798828 688.95703125 246.9413299560547 688.0357666015625 246.9366912841797 C 687.4876098632813 246.9206085205078 687.2227172851563 246.9159698486328 686.8841552734375 246.9159698486328 L 672.0718383789063 246.9159698486328 L 671.3486328125 246.9159698486328 L 670.8880004882813 246.9275054931641 C 669.9782104492188 246.9321441650391 669.0684204101563 247.0150299072266 668.1725463867188 247.1716461181641 C 667.191162109375 247.3328399658203 666.2423706054688 247.6438140869141 665.35546875 248.0929412841797 C 664.464111328125 248.5466766357422 663.6488647460938 249.1385955810547 662.9393920898438 249.8480072021484 C 662.9117431640625 249.8710479736328 662.8724975585938 249.9148101806641 662.6723022460938 250.1267242431641 C 662.4189453125 250.3570709228516 662.1932983398438 250.5873565673828 661.9813232421875 250.8153533935547 C 661.64501953125 251.1470489501953 661.4354858398438 251.3497772216797 661.205078125 251.5893402099609 L 655.9537963867188 256.8314819335938 L 655.7440795898438 257.0364990234375 L 655.5506591796875 257.2276916503906 C 655.3433227539063 257.4257507324219 655.2119750976563 257.5593566894531 655.0669555664063 257.7090148925781 L 650.3798217773438 262.3938903808594 L 650.2645263671875 262.5067749023438 L 650.1448974609375 262.6264953613281 L 649.9307250976563 262.8406677246094 C 649.739501953125 263.0317993164063 649.6405029296875 263.1332092285156 649.5299682617188 263.2553100585938 C 649.1383056640625 263.6375732421875 648.7720947265625 264.0406799316406 648.4312133789063 264.4667663574219 C 648.1019287109375 264.8169250488281 647.834716796875 265.2200012207031 647.6434326171875 265.659912109375 L 647.5905151367188 265.8049926757813 C 647.2451782226563 266.8760681152344 647.2451782226563 268.0253295898438 647.5905151367188 269.0940856933594 L 647.6504516601563 269.2460632324219 C 647.8439331054688 269.6859741210938 648.1088256835938 270.0889892578125 648.4335327148438 270.4391479492188 C 648.7881469726563 270.8721313476563 649.1660766601563 271.2867431640625 649.5643920898438 271.6783142089844 L 649.7349243164063 271.8533935546875 L 649.9329223632813 272.0536804199219 L 650.1494750976563 272.2840576171875 L 650.3798217773438 272.5143737792969 L 655.0900268554688 277.2314147949219 L 655.5506591796875 277.6944274902344 L 655.9284057617188 278.065185546875 L 656.1056518554688 278.2310485839844 L 661.2166137695313 283.3396911621094 C 661.4469604492188 283.5699462890625 661.6773071289063 283.80029296875 662.0044555664063 284.1227416992188 C 662.1793823242188 284.3001708984375 662.3751220703125 284.4935607910156 662.6701049804688 284.7769470214844 L 662.7644653320313 284.8759765625 C 662.8865356445313 285.0049438476563 662.9165649414063 285.0372009277344 662.9739990234375 285.090087890625 C 663.6719360351563 285.7857666015625 664.4757690429688 286.3661804199219 665.3533325195313 286.81298828125 C 666.2447509765625 287.2644348144531 667.1981811523438 287.5754089355469 668.1840209960938 287.7343139648438 C 669.0867919921875 287.8862915039063 670.0013427734375 287.9669494628906 670.9155883789063 287.9761962890625 L 671.3556518554688 287.9876708984375 L 672.0787353515625 287.9876708984375 L 686.8749389648438 287.9876708984375 C 687.2134399414063 287.9876708984375 687.4784545898438 287.9876708984375 688.0612182617188 287.9669494628906 C 688.9685668945313 287.9599914550781 689.8760986328125 287.8816528320313 690.7720336914063 287.736572265625 C 691.755615234375 287.57763671875 692.7067260742188 287.2666931152344 693.595947265625 286.8153076171875 C 694.4895629882813 286.3615417480469 695.3072509765625 285.7696228027344 696.0142822265625 285.0579528808594 C 696.7167358398438 284.3507995605469 697.3087158203125 283.5400390625 697.7647094726563 282.6533203125 C 698.2137451171875 281.761962890625 698.5272216796875 280.8106384277344 698.697509765625 279.824951171875 C 698.842529296875 278.9220581054688 698.9208984375 278.00537109375 698.9277954101563 277.088623046875 C 698.9393310546875 276.6786499023438 698.9463500976563 276.2917175292969 698.9463500976563 275.9209289550781 L 698.9463500976563 258.6994323730469 C 698.9485473632813 258.2525939941406 698.9463500976563 258.056884765625 698.93701171875 257.8011779785156 Z M 686.4142456054688 275.4809875488281 C 685.8845825195313 276.0130615234375 685.0254516601563 276.0130615234375 684.493408203125 275.4832458496094 L 684.4771728515625 275.4994201660156 L 678.3367309570313 269.375 L 672.2032470703125 275.5108947753906 L 672.1801147460938 275.4856262207031 C 672.108642578125 275.5523681640625 672.0328369140625 275.6122436523438 671.949951171875 275.6605834960938 C 671.3141479492188 276.0660095214844 670.4688720703125 275.8794250488281 670.0634155273438 275.2437133789063 C 669.660400390625 274.6080627441406 669.8468017578125 273.7627563476563 670.482666015625 273.3573608398438 L 676.3997802734375 267.4425964355469 L 670.2499389648438 261.3067321777344 L 670.275390625 261.2814331054688 C 669.7455444335938 260.7470703125 669.7501831054688 259.8856201171875 670.2845458984375 259.3581848144531 C 670.8187866210938 258.8284606933594 671.6780395507813 258.8330383300781 672.207763671875 259.3673706054688 C 672.2745361328125 259.4364929199219 672.3344116210938 259.5148010253906 672.3850708007813 259.5976867675781 L 678.3252563476563 265.5285339355469 L 684.3136596679688 259.5401000976563 C 684.3574829101563 259.4756469726563 684.4080810546875 259.4157409667969 684.4635009765625 259.3604431152344 C 684.988525390625 258.8537292480469 685.820068359375 258.8537292480469 686.347412109375 259.3581848144531 C 686.893310546875 259.8764343261719 686.9141235351563 260.7401428222656 686.3936157226563 261.2860107421875 L 686.4095458984375 261.3021850585938 L 680.2644653320313 267.4495544433594 L 686.2186279296875 273.3965148925781 C 686.2876586914063 273.442626953125 686.3543701171875 273.4978332519531 686.4142456054688 273.5601501464844 C 686.9463500976563 274.0897521972656 686.9463500976563 274.9489440917969 686.4142456054688 275.4809875488281 Z" fill="#08eef3" stroke="none" stroke-width="1" stroke-miterlimit="10" stroke-linecap="butt" /></svg>',
                      allowDrawingOutsideViewBox: true,
                    ),
                  ],
                ),
              ),
            ),
          ),
        ],
      );
}
