import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'dart:ui' as ui;
import 'package:esys_flutter_share/esys_flutter_share.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:image_gallery_saver/image_gallery_saver.dart';
import 'package:intl/intl.dart';
import 'package:minilikewallet/home/button_action/button_action.dart';
import 'package:minilikewallet/home/home_component/home_component.dart';
import 'package:minilikewallet/provider/home_provider/home_provider.dart';
import 'package:minilikewallet/provider/home_provider/index_provider/index_provider.dart';
import 'package:minilikewallet/provider/home_provider/send_provider/send_provider.dart';
import 'package:minilikewallet/send/component/send_receipt_component.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:minilikewallet/send/send_confirm.dart';
import 'package:minilikewallet/theme/custom_loading.dart';
import 'package:minilikewallet/theme/widget.dart';
import 'package:modal_progress_hud/modal_progress_hud.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:provider/provider.dart';

class SendReceiptScreen extends StatefulWidget {
  @override
  _SendReceiptScreen createState() => _SendReceiptScreen();
}

class _SendReceiptScreen extends State<SendReceiptScreen> {
  SendProvider _sendProvider;
  HomeProvider _getLikeWallet;
  IndexProvider _indexProvider;
  final format = new DateFormat('dd-MM-yyyy HH:MM');
  DateTime now = DateTime.now();
  GlobalKey _globalKey = new GlobalKey();

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();
    _getLikeWallet = Provider.of<HomeProvider>(context);
    _sendProvider = Provider.of<SendProvider>(context, listen: true);
    _indexProvider = Provider.of<IndexProvider>(context, listen: true);
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return ModalProgressHUD(
      opacity: 0.1,
      inAsyncCall: _sendProvider.getLoading,
      progressIndicator: CustomLoading(),
      child: Container(
        height: 2119.h - 174.h,
        width: 1080.w,
        decoration: BoxDecoration(
          color: Colors.transparent,
          borderRadius: BorderRadius.circular(103.0.h),
        ),
        child: Stack(
          children: [
            Positioned(
                top: -174.h, child: SendReceiptComponent.receiptBackground()),
            Positioned(
                left: -374.58.w,
                bottom: 0.h,
                child: SendReceiptComponent.line()),
            receipt()
          ],
        ),
      ),
    );
  }

  Widget receipt() {
    return Column(
      children: [
        SizedBox(height: 122.h),
        RepaintBoundary(
          key: _globalKey,
          child: Container(
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Stack(
                  children: [
                    SendReceiptComponent.imageReceipt(),
                    Padding(
                      padding: EdgeInsets.only(left: 140.w),
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.center,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [receiptDetail()],
                      ),
                    ),
                  ],
                ),
              ],
            ),
          ),
        ),
        SizedBox(height: 100.h),
        Container(
          padding: EdgeInsets.symmetric(horizontal: 114.h),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              InkWell(
                  onTap: () => share(),
                  child: SendReceiptComponent.buttonShare()),
              // SizedBox(width: 43.w),
              InkWell(
                  onTap: () => _capturePng(),
                  child: SendReceiptComponent.buttonSave()),
            ],
          ),
        ),
        Expanded(child: Container()),
        FlatButton(
            onPressed: () {
              // _getLikeWallet.dispose();
              _getLikeWallet.setSendReceipt(status: false);
              _getLikeWallet.setSendConfirm(status: false);
              _getLikeWallet.setMenu(status: true);
              // _sendProvider.dispose();
              _indexProvider.loadBalance();
            },
            child: ButtonAction.closeMenuWhite()),
        // SizedBox(height: 45.h),
      ],
    );
  }

  Widget receiptDetail() {
    return Container(
      width: 800.w,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          SizedBox(height: 120.h),
          SendReceiptComponent.title(text: 'From'),
          SendReceiptComponent.detail(text: _sendProvider.getNameFOMR),
          SizedBox(height: 40.h),
          SendReceiptComponent.title(text: 'To'),
          SendReceiptComponent.detail(text: _sendProvider.getNameTo),
          SizedBox(height: 40.h),
          SendReceiptComponent.title(text: 'Amount'),
          SendReceiptComponent.detail(
              text: _sendProvider.getAmount.join() + ' LIKE'),
          SizedBox(height: 40.h),
          SendReceiptComponent.title(text: 'Fee'),
          SendReceiptComponent.detail(text: '0'),
          SizedBox(height: 40.h),
          SendReceiptComponent.title(text: 'NOTE :'),
          Container(
            width: 583.w,
            child: SendReceiptComponent.textNote(
                text: _sendProvider.noteController.text),
          ),
          SizedBox(height: 70.h),
          SendReceiptComponent.textBack31(text: 'Transaction Completed!'),
          SendReceiptComponent.textBack31(
              text: DateFormat('dd-MM-yyyy').format(now).toString()),
          SendReceiptComponent.textBack31(
              text: DateFormat('hh:mm a').format(now).toString()),
        ],
      ),
    );
  }

  void share() async {
    _sendProvider.setLoading(status: true);
    RenderRepaintBoundary boundary =
        _globalKey.currentContext.findRenderObject();
    ui.Image image = await boundary.toImage(pixelRatio: 5.0);
    ByteData bytes = await image.toByteData(format: ui.ImageByteFormat.png);
    _sendProvider.setLoading(status: false);
    await Share.file('receipt', '${DateTime.now()}.png',
        bytes.buffer.asUint8List(), 'image/png',
        text: '${DateTime.now()}');
  }

  Future<Uint8List> _capturePng() async {
    _sendProvider.setLoading(status: true);
    RenderRepaintBoundary boundary =
        _globalKey.currentContext.findRenderObject();
    ui.Image image = await boundary.toImage(pixelRatio: 5.0);
    ByteData byteData = await image.toByteData(format: ui.ImageByteFormat.png);
    Uint8List pngBytes = byteData.buffer.asUint8List();
    if (Platform.isAndroid) {
      bool statusPermision = await Permission.storage.isGranted;
      if (statusPermision != true) {
        await Permission.storage.request();
        _capturePng();
      } else {
        // Use plugin [path_provider] to export image to storage
        final result = await ImageGallerySaver.saveImage(
            Uint8List.fromList(pngBytes.buffer.asInt8List()));
        print(result);
        print('png done');
        _sendProvider.setLoading(status: false);
        WidgetTheme.showShortToastGrey('บันทึกเเล้ว');
        return pngBytes;
      }
    } else {
      bool statusPermision = await Permission.photos.isGranted;
      if (statusPermision != true) {
        await Permission.photos.request();
      } else {
        // Use plugin [path_provider] to export image to storage
        final result = await ImageGallerySaver.saveImage(
            Uint8List.fromList(pngBytes.buffer.asInt8List()));
        print(result);
        _sendProvider.setLoading(status: false);
        print('png done');
        return pngBytes;
      }
    }
  }
}
